@@ -37,6 +37,12 @@
 #include "iterator.hpp"
 
 #include "thread/radix_key_codec.hpp"
+#include "thread/thread_load.hpp"
+#include "thread/thread_operators.hpp"
+#include "thread/thread_reduce.hpp"
+#include "thread/thread_scan.hpp"
+#include "thread/thread_search.hpp"
+#include "thread/thread_store.hpp"
 
 #include "warp/warp_reduce.hpp"
 #include "warp/warp_scan.hpp"
@@ -51,14 +57,20 @@
 #include "block/block_scan.hpp"
 #include "block/block_sort.hpp"
 #include "block/block_store.hpp"
+#include "block/config.hpp"
 
 #include "device/device_adjacent_difference.hpp"
+#include "device/device_adjacent_find.hpp"
 #include "device/device_binary_search.hpp"
 #include "device/device_copy.hpp"
+#include "device/device_find_end.hpp"
+#include "device/device_find_first_of.hpp"
 #include "device/device_histogram.hpp"
 #include "device/device_memcpy.hpp"
 #include "device/device_merge.hpp"
 #include "device/device_merge_sort.hpp"
+#include "device/device_nth_element.hpp"
+#include "device/device_partial_sort.hpp"
 #include "device/device_partition.hpp"
 #include "device/device_radix_sort.hpp"
 #include "device/device_reduce.hpp"
@@ -66,6 +78,8 @@
 #include "device/device_run_length_encode.hpp"
 #include "device/device_scan.hpp"
 #include "device/device_scan_by_key.hpp"
+#include "device/device_search.hpp"
+#include "device/device_search_n.hpp"
 #include "device/device_segmented_radix_sort.hpp"
 #include "device/device_segmented_reduce.hpp"
 #include "device/device_segmented_scan.hpp"
