@@ -116,7 +116,7 @@ enum class warp_load_method
 template<
     class T,
     unsigned int ItemsPerThread,
-    unsigned int WarpSize = ::rocprim::device_warp_size(),
+    unsigned int WarpSize = ::rocprim::arch::wavefront::min_size(),
     warp_load_method Method = warp_load_method::warp_load_direct
 >
 class warp_load
@@ -124,7 +124,7 @@ class warp_load
     static_assert(::rocprim::detail::is_power_of_two(WarpSize),
                   "Logical warp size must be a power of two.");
     ROCPRIM_DETAIL_DEVICE_STATIC_ASSERT(
-        WarpSize <= ::rocprim::device_warp_size(),
+        WarpSize <= ::rocprim::arch::wavefront::min_size(),
         "Logical warp size cannot be larger than physical warp size.");
 
 private:
@@ -249,7 +249,7 @@ class warp_load<T, ItemsPerThread, WarpSize, warp_load_method::warp_load_striped
     static_assert(::rocprim::detail::is_power_of_two(WarpSize),
                   "Logical warp size must be a power of two.");
     ROCPRIM_DETAIL_DEVICE_STATIC_ASSERT(
-        WarpSize <= ::rocprim::device_warp_size(),
+        WarpSize <= ::rocprim::arch::wavefront::min_size(),
         "Logical warp size cannot be larger than physical warp size.");
 
 public:
@@ -315,7 +315,7 @@ class warp_load<T, ItemsPerThread, WarpSize, warp_load_method::warp_load_vectori
     static_assert(::rocprim::detail::is_power_of_two(WarpSize),
                   "Logical warp size must be a power of two.");
     ROCPRIM_DETAIL_DEVICE_STATIC_ASSERT(
-        WarpSize <= ::rocprim::device_warp_size(),
+        WarpSize <= ::rocprim::arch::wavefront::min_size(),
         "Logical warp size cannot be larger than physical warp size.");
 
 public:
@@ -390,7 +390,7 @@ class warp_load<T, ItemsPerThread, WarpSize, warp_load_method::warp_load_transpo
     static_assert(::rocprim::detail::is_power_of_two(WarpSize),
                   "Logical warp size must be a power of two.");
     ROCPRIM_DETAIL_DEVICE_STATIC_ASSERT(
-        WarpSize <= ::rocprim::device_warp_size(),
+        WarpSize <= ::rocprim::arch::wavefront::min_size(),
         "Logical warp size cannot be larger than physical warp size.");
 
 private:
